unit ImportV1Database;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, UITypes,
  Dialogs, StdCtrls, ComCtrls;

type
  TFormImport = class(TForm)
    GroupBoxContainer: TGroupBox;
    ButtonClose: TButton;
    ButtonImport: TButton;
    LabelSettingsFileName: TLabel;
    EditSettingsFileName: TEdit;
    ButtonSelectFile: TButton;
    OpenDialog: TOpenDialog;
    ProgressBar: TProgressBar;
    LabelProgress: TLabel;
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonSelectFileClick(Sender: TObject);
    procedure EditSettingsFileNameChange(Sender: TObject);
    procedure ButtonImportClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ImportV1DataBase(FileName: String);
    procedure ImportFolder(FolderName: String; ProjectName: String);
    procedure ImportRegistrations(FolderName: String; ProjectTableName: String);
  end;

var
  FormImport: TFormImport;

implementation

{$R *.dfm}

Uses
    IniFiles,
    Helper3delite,
    AddNewProject,
    AddNewSubVersion,
    Main;

procedure TFormImport.ButtonCloseClick(Sender: TObject);
begin
    Close;
end;

procedure TFormImport.ButtonImportClick(Sender: TObject);
begin
    Screen.Cursor := crAppStart;
    try
        ImportV1DataBase(EditSettingsFileName.Text);
        LabelProgress.Caption := 'Importing done.';
        Screen.Cursor := crDefault;
    except
        on E: Exception do begin
            Screen.Cursor := crDefault;
            MessageDlg('Error: ' + E.Message, mtError, [mbOk], 0);
        end;
    end;
end;

procedure TFormImport.ButtonSelectFileClick(Sender: TObject);
begin
    if OpenDialog.Execute then begin
        EditSettingsFileName.Text := OpenDialog.FileName;
    end;
end;

procedure TFormImport.EditSettingsFileNameChange(Sender: TObject);
begin
    ButtonImport.Enabled := FileExists(EditSettingsFileName.Text);
end;

procedure TFormImport.ImportV1DataBase(FileName: String);
var
    SettingsFile: TIniFile;
    ProjectsFolder: String;
    ProjectFolders: TStringList;
    ProjectNames: TStringList;
    i: Integer;
begin
    SettingsFile := nil;
    ProjectFolders := nil;
    ProjectNames := nil;

    SettingsFile := TIniFile.Create(FileName);
    try
        ProjectsFolder := SettingsFile.ReadString('Settings', 'DataFolder', '');
        if NOT DirectoryExists(ProjectsFolder) then begin
            MessageDlg('Error: Projects folder does not exists!', mtError, [mbOk], 0);
            Exit;
        end;
        if NOT FileExists(ProjectsFolder + 'ProjectNames.txt') then begin
            MessageDlg('Error: "ProjectNames.txt" does not exists in projects folder!', mtError, [mbOk], 0);
            Exit;
        end;
        ProjectNames := TStringList.Create;
        ProjectNames.LoadFromFile(ProjectsFolder + 'ProjectNames.txt');
        ProjectFolders := TStringList.Create;
        for i := 0 to ProjectNames.Count - 1 do begin
            ProjectFolders.Append(ProjectsFolder + ProjectNames[i] + '\');
        end;
        for i := 0 to ProjectFolders.Count - 1 do begin
            ImportFolder(ProjectFolders[i], ProjectNames[i]);
        end;
        //* Success
    finally
        if Assigned(SettingsFile) then begin
            FreeAndNil(SettingsFile);
        end;
        if Assigned(ProjectFolders) then begin
            FreeAndNil(ProjectFolders);
        end;
        if Assigned(ProjectNames) then begin
            FreeAndNil(ProjectNames);
        end;
    end;
end;

procedure TFormImport.ImportFolder(FolderName: String; ProjectName: String);
var
    SettingsFile: TIniFile;
    RegistrationSeed: String;
    ProjectTableName: String;
    SubVersionSettings: TStringList;
    SubVersionNames: TStringList;
    SubVersionSeeds: TStringList;
    i: Integer;
    ProjectID: Integer;
begin
    LabelProgress.Caption := 'Importing project: ' + ProjectName + '...';
    Application.ProcessMessages;
    SettingsFile := nil;
    SubVersionSettings := nil;
    SubVersionNames := nil;
    SubVersionSeeds := nil;
    SettingsFile := TIniFile.Create(FolderName + 'Settings.ini');
    try
        RegistrationSeed := SettingsFile.ReadString('Settings', 'RegSeed', '0');
        ProjectTableName := StrConvertToSQLName(ProjectName);

        ProjectID := FormAddNewProject.SQLAddNewProject(ProjectName, ProjectTableName, RegistrationSeed, '', 'http://www.mywebpage.com');

        SubVersionSettings := TStringList.Create;
        SubVersionSettings.LoadFromFile(FolderName + 'SubversionData.txt');
        SubVersionNames := TStringList.Create;
        SubVersionSeeds := TStringList.Create;
        for i := 0 to SubVersionSettings.Count - 1 do begin
            if Copy(SubVersionSettings[i], 1, 1) = '#' then begin
                SubVersionNames.Append(Copy(SubVersionSettings[i], 2, Length(SubVersionSettings[i])));
            end;
            if Copy(SubVersionSettings[i], 1, 1) = '>' then begin
                SubVersionSeeds.Append(Copy(SubVersionSettings[i], 2, Length(SubVersionSettings[i])));
            end;
        end;

        for i := 0 to SubVersionNames.Count - 1 do begin
            FormNewSubVersion.SQLAddNewProjectSubVersion(ProjectID, SubVersionNames[i], SubVersionSeeds[i]);
        end;

        ImportRegistrations(FolderName, ProjectTableName);

    finally
        if Assigned(SettingsFile) then begin
            FreeAndNil(SettingsFile);
        end;
        if Assigned(SubVersionSettings) then begin
            FreeAndNil(SubVersionSettings);
        end;
        if Assigned(SubVersionNames) then begin
            FreeAndNil(SubVersionNames);
        end;
        if Assigned(SubVersionSeeds) then begin
            FreeAndNil(SubVersionSeeds);
        end;
    end;
end;


procedure TFormImport.ImportRegistrations(FolderName: String; ProjectTableName: String);
var
    SettingsFile: TIniFile;
    f: TSearchRec;
    NoFiles: Integer;
    i: Integer;
    FileName: String;
    UserName: String;
    UserEmail: String;
    RegistrationTime: TDateTime;
    CommentsFileName: String;
    Comments: TStringList;
begin
    try
        NoFiles := 0;
        if FindFirst(FolderName + '*.uri', faAnyFile, f) = 0 then
        repeat
        	Inc(NoFiles);
        until findNext(f) <> 0;
        FindClose(f);
        ProgressBar.Max := NoFiles;
        ProgressBar.Position := 0;
        for i := 1 to NoFiles do begin
            FileName := FolderName + IntToStr(100000 + i) + '.uri';
            if FileExists(FileName) then begin
                SettingsFile := TIniFile.Create(FileName);
                UserName := SettingsFile.ReadString('RegInfo', 'Name', 'NoName');
                UserEmail := SettingsFile.ReadString('RegInfo', 'EMail', 'NoEMail');
                try
                    RegistrationTime := SettingsFile.ReadDateTime('RegInfo', 'Date', GetFileModDate(FileName));
                except
                    RegistrationTime := GetFileModDate(FileName);
                end;
            end else begin
                UserName := 'Deleted registration';
                UserEmail := '';
                RegistrationTime := 0;
            end;
            Comments := TStringList.Create;
            CommentsFileName := FolderName + IntToStr(100000 + i) + '.email';
            if FileExists(CommentsFileName) then begin
                Comments.LoadFromFile(CommentsFileName);
            end;
            FormTool.SQLAddNewProjectRegistration(ProjectTableName, UserName, UserEmail, RegistrationTime, Comments.Text);
            FreeAndNil(Comments);
            FreeAndNil(SettingsFile);
            ProgressBar.StepIt;
            Application.ProcessMessages;
        end;
    except
        on E: Exception do MessageDlg('Error: ' + E.Message, mtError, [mbOk], 0);
    end;
end;


end.
