unit ProjectManager;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, UITypes,
  Dialogs, StdCtrls, ComCtrls, DB, ADODB, ImgList, System.ImageList;

type
  TFormProjectManager = class(TForm)
    GroupBoxContainer: TGroupBox;
    ButtonClose: TButton;
    ListViewProjects: TListView;
    ListViewSubVersions: TListView;
    ButtonAddNewProject: TButton;
    ButtonDeleteProject: TButton;
    ButtonAddNewSubversion: TButton;
    ButtonDeleteSubVersion: TButton;
    DataSource: TDataSource;
    ADOQuery: TADOQuery;
    ADOConnection: TADOConnection;
    ImageListProjectIcons: TImageList;
    ButtonEditProject: TButton;
    ButtonEditEMailTemplates: TButton;
    procedure ButtonCloseClick(Sender: TObject);
    procedure ButtonAddNewProjectClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ButtonAddNewSubversionClick(Sender: TObject);
    procedure ListViewProjectsChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure ListViewSubVersionsChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure ButtonDeleteProjectClick(Sender: TObject);
    procedure ButtonDeleteSubVersionClick(Sender: TObject);
    procedure ListViewProjectsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure ButtonEditProjectClick(Sender: TObject);
    procedure ListViewSubVersionsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure ButtonEditEMailTemplatesClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ReloadProjects;
    procedure ReloadSubVersions;
    procedure UpdateControlsStatus;
    procedure SQLDeleteProject(ProjectID: Integer; ProjectTableName: String);
    procedure SQLDeleteProjectSubVersion(SubVersionName: String);
    procedure SQLLoadProjectEMailTemplates(ProjectID: Integer; var EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError: String);
    function SQLSetEMailTemplates(ProjectID: Integer; EMailTemplateRegistrationText, EMailTemplateVersionCodeText, EMailTemplateVersionCodeErrorText: String): Boolean;

  end;

var
  FormProjectManager: TFormProjectManager;

implementation

{$R *.dfm}

Uses
    Main,
    AddNewProject,
    Settings,
    AddNewSubVersion,
    EditProject,
    EditEMailTemplates;

procedure TFormProjectManager.ButtonEditEMailTemplatesClick(Sender: TObject);
var
    EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError: String;
begin
    SQLLoadProjectEMailTemplates(StrToInt(ListViewProjects.Selected.SubItems[0]), EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError);
    if (EMailTemplateRegistration = '')
    AND (EMailTemplateVersionCode = '')
    then begin
        FormTool.SQLLoadDefaultEMailTemplates(EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError);
    end;
    FormEditEMailTemplates.MemoRegistrationEMailTemplate.Text := EMailTemplateRegistration;
    FormEditEMailTemplates.MemoVersionCodeEMailTemplate.Text := EMailTemplateVersionCode;
    FormEditEMailTemplates.MemoVersionCodeErrorEMailTemplate.Text := EMailTemplateVersionCodeError;
    case FormEditEMailTemplates.ShowModal of
        mrOk: begin
            if SQLSetEMailTemplates(StrToInt(ListViewProjects.Selected.SubItems[0]), FormEditEMailTemplates.MemoRegistrationEMailTemplate.Text, FormEditEMailTemplates.MemoVersionCodeEMailTemplate.Text, FormEditEMailTemplates.MemoVersionCodeErrorEMailTemplate.Text) then begin
                MessageDlg('New e-mail templates saved.', mtInformation, [mbOk], 0);
            end;
        end;
    end;
end;

procedure TFormProjectManager.ButtonAddNewProjectClick(Sender: TObject);
begin
    FormAddNewProject.ClearIcon;
    FormAddNewProject.ShowModal;
    ReloadProjects;
    UpdateControlsStatus;
end;

procedure TFormProjectManager.ButtonAddNewSubversionClick(Sender: TObject);
begin
    if Assigned(ListViewProjects.Selected) then begin
        FormNewSubVersion.ProjectID := StrToInt(ListViewProjects.Selected.SubItems[0]);
        if ListViewSubVersions.Items.Count > 0 then begin
            FormNewSubVersion.EditName.Text := ListViewSubVersions.Items.Item[ListViewSubVersions.Items.Count - 1].Caption;
            FormNewSubVersion.EditRegSeed.Text := ListViewSubVersions.Items.Item[ListViewSubVersions.Items.Count - 1].SubItems[0];
        end;
        FormNewSubVersion.ShowModal;
        ReloadSubVersions;
        ListViewSubVersions.AlphaSort;
        UpdateControlsStatus;
    end;
end;

procedure TFormProjectManager.ButtonCloseClick(Sender: TObject);
begin
    Close;
end;

procedure TFormProjectManager.ButtonDeleteProjectClick(Sender: TObject);
var
    DialogResult: Integer;
begin
    Screen.Cursor := crAppStart;
    try
        DialogResult := MessageDlg('Are you sure you want to delete project "' + ListViewProjects.Selected.Caption + '".', mtConfirmation, [mbYes, mbCancel], 0);
        if DialogResult = mrYes then begin
            SQLDeleteProject(StrToInt(ListViewProjects.Selected.SubItems[0]), ListViewProjects.Selected.SubItems[1]);
            Screen.Cursor := crDefault;
            MessageDlg('Successfully deleted project "' + ListViewProjects.Selected.Caption + '".', mtConfirmation, [mbOk], 0);
        end;
    except
        on E: Exception do begin
            Screen.Cursor := crDefault;
            MessageDlg('Error: ' + E.Message, mtError, [mbOk], 0);
        end;
    end;
    ReloadProjects;
end;

procedure TFormProjectManager.ButtonDeleteSubVersionClick(Sender: TObject);
var
    DialogResult: Integer;
begin
    Screen.Cursor := crAppStart;
    try
        DialogResult := MessageDlg('Are you sure you want to delete sub version "' + ListViewSubVersions.Selected.Caption + '".', mtConfirmation, [mbYes, mbCancel], 0);
        if DialogResult = mrYes then begin
            SQLDeleteProjectSubVersion(ListViewSubVersions.Selected.Caption);
            Screen.Cursor := crDefault;
            MessageDlg('Successfully deleted sub version "' + ListViewSubVersions.Selected.Caption + '".', mtConfirmation, [mbOk], 0);
        end;
    except
        on E: Exception do begin
            Screen.Cursor := crDefault;
            MessageDlg('Error: ' + E.Message, mtError, [mbOk], 0);
        end;
    end;
    ReloadSubVersions;
    ListViewSubVersions.AlphaSort;
end;

procedure TFormProjectManager.ButtonEditProjectClick(Sender: TObject);
begin
    {
    FormEditProject.ClearIcon;
    FormEditProject.EditName.Text := ListViewProjects.Selected.Caption;
    FormEditProject.EditTableName.Text := ListViewProjects.Selected.SubItems[1];
    FormEditProject.PreviousProjectTableName := ListViewProjects.Selected.SubItems[1];
    FormEditProject.EditRegSeed.Text := ListViewProjects.Selected.SubItems[2];
    }
    FormEditProject.ProjectID := StrToInt(ListViewProjects.Selected.SubItems[0]);
    FormEditProject.PreviousProjectTableName := ListViewProjects.Selected.SubItems[1];
    FormEditProject.LoadData;
    FormEditProject.ShowModal;
    ReloadProjects;
    UpdateControlsStatus;
end;

procedure TFormProjectManager.FormCreate(Sender: TObject);
begin
    SetColumns(ListViewProjects);
end;

procedure TFormProjectManager.FormShow(Sender: TObject);
begin
    UpdateControlsStatus;
    ReloadProjects;
end;

procedure TFormProjectManager.ListViewProjectsChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
    UpdateControlsStatus;
end;

procedure TFormProjectManager.ListViewProjectsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
    if Selected then begin
        ReloadSubVersions;
        ListViewSubVersions.AlphaSort;
    end else begin
        ListViewSubVersions.Clear;
    end;
end;

procedure TFormProjectManager.ListViewSubVersionsChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
    UpdateControlsStatus;
end;

procedure TFormProjectManager.ListViewSubVersionsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
    Item1Text: String;
    Item2Text: String;
    Item1MajorVersion: Integer;
    Item1MinorVersion: Integer;
    Item1Release: Integer;
    Item1Build: Integer;
    Item2MajorVersion: Integer;
    Item2MinorVersion: Integer;
    Item2Release: Integer;
    Item2Build: Integer;
begin
    Item1Text := Item1.Caption;
    Item2Text := Item2.Caption;

    try

        Item1MajorVersion := StrToInt(Copy(Item1Text, 0, Pos('.', Item1Text) - 1));
        Item1Text := Copy(Item1Text, Pos('.', Item1Text) + 1, Length(Item1Text));
        try
            Item1MinorVersion := StrToInt(Copy(Item1Text, 0, Pos('.', Item1Text) - 1));
            Item1Text := Copy(Item1Text, Pos('.', Item1Text) + 1, Length(Item1Text));
        except
            Item1MinorVersion := 0;
        end;
        try
            Item1Release := StrToInt(Copy(Item1Text, 0, Pos('.', Item1Text) - 1));
            Item1Text := Copy(Item1Text, Pos('.', Item1Text) + 1, Length(Item1Text));
        except
            Item1Release := 0;
        end;
        try
            Item1Build := StrToInt(Copy(Item1Text, 0, Length(Item1Text)));
        except
            Item1Build := 0;
        end;

        Item2MajorVersion := StrToInt(Copy(Item2Text, 0, Pos('.', Item2Text) - 1));
        Item2Text := Copy(Item2Text, Pos('.', Item2Text) + 1, Length(Item2Text));
        try
            Item2MinorVersion := StrToInt(Copy(Item2Text, 0, Pos('.', Item2Text) - 1));
            Item2Text := Copy(Item2Text, Pos('.', Item2Text) + 1, Length(Item2Text));
        except
            Item2MinorVersion := 0;
        end;
        try
            Item2Release := StrToInt(Copy(Item2Text, 0, Pos('.', Item2Text) - 1));
            Item2Text := Copy(Item2Text, Pos('.', Item2Text) + 1, Length(Item2Text));
        except
            Item2Release := 0;
        end;
        try
            Item2Build := StrToInt(Copy(Item2Text, 0, Length(Item2Text)));
        except
            Item2Build := 0;
        end;

        if Item1MajorVersion = Item2MajorVersion then begin
            if Item1MinorVersion = Item2MinorVersion then begin
                if Item1Release = Item2Release then begin
                    if Item1Build = Item2Build then begin
                        Compare := 0;
                        Exit;
                    end else begin
                        if Item1Build > Item2Build then begin
                            Compare := 1;
                            Exit;
                        end else begin
                            Compare := -1;
                            Exit;
                        end;
                    end;
                end else begin
                    if Item1Release > Item2Release then begin
                        Compare := 1;
                        Exit;
                    end else begin
                        Compare := -1;
                        Exit;
                    end;
                end;
            end else begin
                if Item1MinorVersion > Item2MinorVersion then begin
                    Compare := 1;
                    Exit;
                end else begin
                    Compare := -1;
                    Exit;
                end;
            end;
        end else begin
            if Item1MajorVersion > Item2MajorVersion then begin
                Compare := 1;
                Exit;
            end else begin
                Compare := -1;
                Exit;
            end;
        end;
    except
        Compare := 0;
    end;
end;

procedure TFormProjectManager.ReloadProjects;
var
    Icon: TIcon;
    IconBytes: TBytes;
    IconStream: TMemoryStream;
begin

    ADOConnection.Close;

    ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;

    ADOQuery.Close;
    ADOQuery.SQL.Clear;

    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    //* Get all data
    ADOQuery.SQL.Add('SELECT * FROM ' + TN_PROJECTS);
    ADOQuery.Open;

    ListViewProjects.Clear;
    ImageListProjectIcons.Clear;
    ImageListProjectIcons.BkColor := ListViewProjects.Color;

    DataSource.DataSet.First;
    while NOT DataSource.DataSet.Eof do begin
        with ListViewProjects.Items.Add do begin
            Caption := DataSource.DataSet.FieldByName(CN_PROJECTNAME).AsWideString;
            SubItems.Add(DataSource.DataSet.FieldByName(CN_PROJECT_ID).AsWideString);
            SubItems.Add(DataSource.DataSet.FieldByName(CN_PROJECT_TABLENAME).AsString);
            SubItems.Add(DataSource.DataSet.FieldByName(CN_PROJECT_REG_SEED).AsString);
            IconBytes := DataSource.DataSet.FieldByName(CN_PROJECT_ICON).AsBytes;
            if Length(IconBytes) > 0 then begin
                Icon := TIcon.Create;
                IconStream := TMemoryStream.Create;
                IconStream.Write(Pointer(IconBytes)^, Length(IconBytes));
                IconStream.Seek(0, soBeginning);
                Icon.LoadFromStream(IconStream);
                ImageIndex := ImageListProjectIcons.AddIcon(Icon);
                FreeAndNil(IconStream);
                FreeAndNil(Icon);
            end else begin
                ImageIndex := -1;
            end;
        end;
        DataSource.DataSet.Next;
    end;

    ADOQuery.Close;
    ADOConnection.Close;
end;

procedure TFormProjectManager.ReloadSubVersions;
var
    ProjectID: String;
begin
    ProjectID := ListViewProjects.Selected.SubItems[0];

    ADOConnection.Close;

    ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;

    ADOQuery.Close;
    ADOQuery.SQL.Clear;
    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    //* Get all data
    ADOQuery.SQL.Add('SELECT * FROM ' + TN_PROJECTS_SUBVERSION + ' WHERE ' + CN_PROJECT_ID + ' = ' + ProjectID + ' ORDER BY ' + CN_SUBVERSION_NAME);
    ADOQuery.Open;

    ListViewSubVersions.Clear;

    DataSource.DataSet.First;
    while NOT DataSource.DataSet.Eof do begin
        with ListViewSubVersions.Items.Add do begin
            Caption := DataSource.DataSet.FieldByName(CN_SUBVERSION_NAME).AsWideString;
            SubItems.Add(DataSource.DataSet.FieldByName(CN_SUBVERSION_VERCODE_SEED).AsWideString);
        end;
        DataSource.DataSet.Next;
    end;

    ADOQuery.Close;
    ADOConnection.Close;
end;

procedure TFormProjectManager.UpdateControlsStatus;
begin
    if Assigned(ListViewProjects.Selected) then begin
        ButtonDeleteProject.Enabled := True;
        ButtonEditProject.Enabled := True;
        ButtonEditEMailTemplates.Enabled := True;
    end else begin
        ButtonDeleteProject.Enabled := False;
        ButtonEditProject.Enabled := False;
        ButtonEditEMailTemplates.Enabled := False;
    end;
    if Assigned(ListViewProjects.Selected) then begin
        ButtonAddNewSubversion.Enabled := True;
    end else begin
        ButtonAddNewSubversion.Enabled := False;
    end;
    if Assigned(ListViewSubVersions.Selected) then begin
        ButtonDeleteSubVersion.Enabled := True;
    end else begin
        ButtonDeleteSubVersion.Enabled := False;
    end;
end;

procedure TFormProjectManager.SQLDeleteProject(ProjectID: Integer; ProjectTableName: String);
begin
    ADOConnection.Close;

    ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;
    ADOQuery.SQL.Clear;
    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    ADOQuery.SQL.Add('DELETE FROM ' + TN_PROJECTS + ' WHERE ' + CN_PROJECT_ID + ' = ' + IntToStr(ProjectID));
    ADOQuery.ExecSql;

    ADOQuery.SQL.Clear;
    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    ADOQuery.SQL.Add('DELETE FROM ' + TN_PROJECTS_SUBVERSION + ' WHERE ' + CN_PROJECT_ID + ' = ' + IntToStr(ProjectID));
    ADOQuery.ExecSql;

    ADOQuery.SQL.Clear;
    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    ADOQuery.SQL.Add('DROP TABLE "' + ProjectTableName + '"');
    ADOQuery.ExecSql;

    ADOConnection.Close;
end;

procedure TFormProjectManager.SQLDeleteProjectSubVersion(SubVersionName: String);
begin
    ADOConnection.Close;

    ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;
    ADOQuery.SQL.Clear;
    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    ADOQuery.SQL.Add('DELETE FROM ' + TN_PROJECTS_SUBVERSION + ' WHERE ' + CN_SUBVERSION_NAME + ' = ''' + SubVersionName + '''');
    ADOQuery.ExecSql;

    ADOConnection.Close;
end;

procedure TFormProjectManager.SQLLoadProjectEMailTemplates(ProjectID: Integer; var EMailTemplateRegistration, EMailTemplateVersionCode, EMailTemplateVersionCodeError: String);
begin

    ADOConnection.Close;

    ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;

    ADOQuery.Close;
    ADOQuery.SQL.Clear;

    //* Set to use our new database (specified by name)
    ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');
    //* Get all data
    ADOQuery.SQL.Add('SELECT * FROM ' + TN_PROJECTS + ' WHERE ' + CN_PROJECT_ID + ' = ' + IntToStr(ProjectID));
    ADOQuery.Open;

    DataSource.DataSet.First;
    while NOT DataSource.DataSet.Eof do begin
        try
            EMailTemplateRegistration := DataSource.DataSet.FieldByName(CN_PROJECT_EMAIL_TEMPLATE_REGISTRATION).AsWideString;
        except
            EMailTemplateRegistration := '';
        end;
        try
            EMailTemplateVersionCode := DataSource.DataSet.FieldByName(CN_PROJECT_EMAIL_TEMPLATE_VERSIONCODE).AsWideString;
        except
            EMailTemplateVersionCode := '';
        end;
        try
            EMailTemplateVersionCodeError := DataSource.DataSet.FieldByName(CN_PROJECT_EMAIL_TEMPLATE_VERSIONCODE_ERROR).AsWideString;
        except
            EMailTemplateVersionCodeError := '';
        end;
        DataSource.DataSet.Next;
    end;

    ADOQuery.Close;

    ADOConnection.Close;
end;

function TFormProjectManager.SQLSetEMailTemplates(ProjectID: Integer; EMailTemplateRegistrationText, EMailTemplateVersionCodeText, EMailTemplateVersionCodeErrorText: String): Boolean;
begin

    //Result := False;

    Screen.Cursor := crAppStart;
    try
        ADOConnection.Close;

        ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;

        ADOQuery.SQL.Clear;

        //* Set to use our new database (specified by name)
        ADOQuery.SQL.Add('USE ' + FormSettings.EditDatabaseNameToUse.Text + ';');

        ADOQuery.SQL.Add('UPDATE ' + TN_PROJECTS);
        ADOQuery.SQL.Add('SET ' + CN_PROJECT_EMAIL_TEMPLATE_REGISTRATION + ' = :a,');
        ADOQuery.SQL.Add(CN_PROJECT_EMAIL_TEMPLATE_VERSIONCODE + ' = :b,');
        ADOQuery.SQL.Add(CN_PROJECT_EMAIL_TEMPLATE_VERSIONCODE_ERROR + ' = :c');
        ADOQuery.SQL.Add('WHERE ' + CN_PROJECT_ID + ' = ' + IntToStr(ProjectID));
        ADOQuery.Parameters.ParamByName('a').DataType := ftWideString;
        ADOQuery.Parameters.ParamByName('a').Value := EMailTemplateRegistrationText;
        ADOQuery.Parameters.ParamByName('b').DataType := ftWideString;
        ADOQuery.Parameters.ParamByName('b').Value := EMailTemplateVersionCodeText;
        ADOQuery.Parameters.ParamByName('c').DataType := ftWideString;
        ADOQuery.Parameters.ParamByName('c').Value := EMailTemplateVersionCodeErrorText;

        ADOQuery.ExecSql;

        ADOQuery.SQL.Clear;

        ADOConnection.Close;

        Result := True;
    finally
        Screen.Cursor := crDefault;
    end;

end;

end.
