unit QueryEmails;

interface

Uses
    SysUtils,
    Classes,
    UITypes,
    Dialogs,
    IDPOP3,
    IDSSL,
    IdSSLOpenSSL,
    IdExplicitTLSClientServerBase,
    IdMessage,
    IdIMAP4,
    IdText,
    //EmailAccountSettings,
    {NotificationSettings}
    IdComponent,
    IdTCPConnection,
    IdTCPClient,
    IdMessageClient,
    IdSMTPBase,
    IdSMTP,
    IdBaseComponent,
    Generics.Collections;

type
    TServerAccountType = (asPOP3, asIMAP, asSMTP);
    TAccountSecurity = (asNone, asSSLTLS);

type
    TAccountItemData = class
        GUID: TGUID;
        //FileName: String;
        ItemName: String;
        Active: Boolean;
        HostName: String;
        Port: Integer;
        LoginName: String;
        Password: String;
        Security: TAccountSecurity;
        SSLMethod: TIdSSLVersion;
        ServerType: TServerAccountType;
    end;
    PAccountItemData = ^TAccountItemData;

type
    TMessageHeader = record
        SenderName: String;
        SenderAddress: String;
        Subject: String;
    end;

type
    TMessageHeaders = Array of TMessageHeader;

    function TestConnectionPOP3(AccountItemData: TAccountItemData): Boolean;
    function TestConnectionIMAP(AccountItemData: TAccountItemData): Boolean;
    function TestConnectionSMTP(AccountItemData: TAccountItemData): Boolean;

    function QueryNewEmailsPOP3(AccountItemData: TAccountItemData; var MessageHeaders: TMessageHeaders): Boolean; overload;
    function QueryNewEmailsIMAP(AccountItemData: TAccountItemData; var MessageHeaders: TMessageHeaders): Boolean; overload;

    procedure SendEmail(AccountItemData: TAccountItemData; From: String; Recipient: String; Subject: String; Body: String);

    function QueryNewEmailsPOP3(AccountItemData: TAccountItemData; Emails: TList<TIdMessage>): Boolean; overload;
    function QueryNewEmailsIMAP(AccountItemData: TAccountItemData; Emails: TList<TIdMessage>): Boolean; overload;

    //procedure ParseEmails;

implementation

Uses
    Main,
    StrUtils,
    Windows,
    ShellAPI{,
    SendEMail};

function TestConnectionPOP3(AccountItemData: TAccountItemData): Boolean;
var
    POP3: TIDPOP3;
    SSL: TIdSSLIOHandlerSocketOpenSSL;
    //MsgCnt: Integer;
    //i: Integer;
    //Msg: TIdMessage;
    //Text: String;
begin
    //Result := False;
    SSL := nil;
    POP3 := TIDPOP3.Create;
    if AccountItemData.Security = asSSLTLS then begin
        SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
    end;
    try
        //* Params
        POP3.ConnectTimeout := 10000;
        POP3.ReadTimeout := 10000;
        //* Server & login
        POP3.Host := AccountItemData.HostName;
        POP3.Port := AccountItemData.Port;
        POP3.Username := AccountItemData.LoginName;
        POP3.Password := AccountItemData.Password;
        //* SSL
        if AccountItemData.Security = asSSLTLS then begin
            SSL.Host := POP3.Host;
            SSL.Port := POP3.Port;
            SSL.Destination := SSL.Host + ':' + IntToStr(SSL.Port);
            SSL.SSLOptions.Method := AccountItemData.SSLMethod;//sslvSSLv3; {good} // sslvTLSv1_2 for 3delite.hu
            POP3.IOHandler := SSL;
            POP3.UseTLS := utUseImplicitTLS;
        end;
        // Try to connect
        POP3.Connect;
        POP3.DisconnectNotifyPeer;
        Result := True;
    finally
        if Assigned(SSL) then begin
            FreeAndNil(SSL);
        end;
        FreeAndNil(POP3);
    end;
end;

function QueryNewEmailsPOP3(AccountItemData: TAccountItemData; var MessageHeaders: TMessageHeaders): Boolean;
var
    POP3: TIDPOP3;
    SSL: TIdSSLIOHandlerSocketOpenSSL;
    MsgCnt: Integer;
    i: Integer;
    Msg: TIdMessage;
    //Text: String;
begin
    Result := False;
    SetLength(MessageHeaders, 0);
    SSL := nil;
    POP3 := TIDPOP3.Create;
    if AccountItemData.Security = asSSLTLS then begin
        SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
    end;
    try
        try
            //* Params
            POP3.ConnectTimeout := 10000;
            POP3.ReadTimeout := 10000;
            //* Server & login
            POP3.Host := AccountItemData.HostName;
            POP3.Port := AccountItemData.Port;
            POP3.Username := AccountItemData.LoginName;
            POP3.Password := AccountItemData.Password;
            //* SSL
            if AccountItemData.Security = asSSLTLS then begin
                SSL.Host := POP3.Host;
                SSL.Port := POP3.Port;
                SSL.Destination := SSL.Host + ':' + IntToStr(SSL.Port);
                SSL.SSLOptions.Method := AccountItemData.SSLMethod; //sslvSSLv3; // sslvTLSv1_2
                POP3.IOHandler := SSL;
                POP3.UseTLS := utUseImplicitTLS;
            end;
            // Try to connect
            POP3.Connect;
            //* Are there new messages?
            MsgCnt := POP3.CheckMessages;
            SetLength(MessageHeaders, MsgCnt);
            //* Parse the message headers
            for i := 1 to MsgCnt do begin
                Msg := TIdMessage.Create;
                try
                    POP3.RetrieveHeader(i, Msg);
                    //* Do something with it
                    MessageHeaders[i - 1].SenderName := Msg.From.Name;
                    MessageHeaders[i - 1].SenderAddress := Msg.From.Address;
                    MessageHeaders[i - 1].Subject := Msg.Subject;
                finally
                    FreeAndNil(Msg);
                end;
            end;
            POP3.DisconnectNotifyPeer;
            if Length(MessageHeaders) > 0 then begin
                Result := True;
            end;
            //* DEBUG
            //Showmessage(Text);
        finally
            if Assigned(SSL) then begin
                FreeAndNil(SSL);
            end;
            FreeAndNil(POP3);
        end;
    except
        //* ???
    end;
end;

function TestConnectionIMAP(AccountItemData: TAccountItemData): Boolean;
var
    IMAPClient: TIdIMAP4;
    OpenSSLHandler: TIdSSLIOHandlerSocketOpenSSL;
begin
    Result := False;
    OpenSSLHandler := nil;
    IMAPClient := TIdIMAP4.Create(nil);
    try
        if AccountItemData.Security = asSSLTLS then begin
            OpenSSLHandler := TIdSSLIOHandlerSocketOpenSSL.Create(nil);
        end;
        try
            if AccountItemData.Security = asSSLTLS then begin
                OpenSSLHandler.SSLOptions.Method := AccountItemData.SSLMethod;
                IMAPClient.IOHandler := OpenSSLHandler;
                IMAPClient.UseTLS := utUseImplicitTLS;
            end;
            IMAPClient.Host := AccountItemData.HostName;
            IMAPClient.Port := AccountItemData.Port;
            IMAPClient.Username := AccountItemData.LoginName;
            IMAPClient.Password := AccountItemData.Password;
            IMAPClient.Connect;
            try
                if IMAPClient.SelectMailBox('INBOX') then begin
                    Result := True;
                end;
            finally
                IMAPClient.Disconnect;
            end;
        finally
            if Assigned(OpenSSLHandler) then begin
                FreeAndNil(OpenSSLHandler);
            end;
        end;
    finally
        FreeAndNil(IMAPClient);
    end;
end;

function QueryNewEmailsIMAP(AccountItemData: TAccountItemData; var MessageHeaders: TMessageHeaders): Boolean;
var
    IMAPClient: TIdIMAP4;
    OpenSSLHandler: TIdSSLIOHandlerSocketOpenSSL;
    MsgIndex: Integer;
    MsgObject: TIdMessage;
    //PartIndex: Integer;
begin
    Result := False;
    SetLength(MessageHeaders, 0);
    OpenSSLHandler := nil;
    IMAPClient := TIdIMAP4.Create(nil);
    try
        if AccountItemData.Security = asSSLTLS then begin
            OpenSSLHandler := TIdSSLIOHandlerSocketOpenSSL.Create(nil);
        end;
        try
            if AccountItemData.Security = asSSLTLS then begin
                OpenSSLHandler.SSLOptions.Method := sslvSSLv3;
                IMAPClient.IOHandler := OpenSSLHandler;
                IMAPClient.UseTLS := utUseImplicitTLS;
            end;
            IMAPClient.Host := AccountItemData.HostName;
            IMAPClient.Port := AccountItemData.Port;
            IMAPClient.Username := AccountItemData.LoginName;
            IMAPClient.Password := AccountItemData.Password;
            IMAPClient.Connect;
            try
                if IMAPClient.SelectMailBox('INBOX') then begin
                    SetLength(MessageHeaders, IMAPClient.MailBox.TotalMsgs);
                    for MsgIndex := 1 to IMAPClient.MailBox.TotalMsgs do begin
                        MsgObject := TIdMessage.Create(nil);
                        try
                            IMAPClient.RetrieveHeader(MsgIndex, MsgObject);
                            //* Do something with it
                            MessageHeaders[MsgIndex - 1].SenderName := MsgObject.From.Name;
                            MessageHeaders[MsgIndex - 1].SenderAddress := MsgObject.From.Address;
                            MessageHeaders[MsgIndex - 1].Subject := MsgObject.Subject;
                        finally
                            MsgObject.Free;
                        end;
                    end;
                    if Length(MessageHeaders) > 0 then begin
                        Result := True;
                    end;
                end;
            finally
                IMAPClient.Disconnect;
            end;
        finally
            if Assigned(OpenSSLHandler) then begin
                FreeAndNil(OpenSSLHandler);
            end;
        end;
    finally
        FreeAndNil(IMAPClient);
    end;
end;

(*
procedure ParseEmails;
var
    i, k: Integer;
    MessageHeaders: TMessageHeaders;
    Detected: Boolean;
    BalloonText: String;
    DetectedBalloonMailCount: Integer;
    DetectedMailCount: Integer;
    NotifyItems: TNotifyItems;
    AlreadyPlayingAudio: Boolean;

    procedure CheckNotifications(NotifyItemData: TNotifyItemData);
    var
        j: Integer;
        MailSenderName: String;
    begin
        for j := Low(MessageHeaders) to High(MessageHeaders) do begin
            Detected := False;
            if (NotifyItemData.Subject <> '')
            AND ContainsText(MessageHeaders[j].Subject, NotifyItemData.Subject)
            then begin
                Detected := True;
            end;
            if (NotifyItemData.Sender <> '')
            AND (ContainsText(MessageHeaders[j].SenderName, NotifyItemData.Sender) OR ContainsText(MessageHeaders[j].SenderAddress, NotifyItemData.Sender))
            then begin
                Detected := True;
            end;
            if (NotifyItemData.Subject = '')
            AND (NotifyItemData.Sender = '')
            then begin
                Detected := True;
            end;
            if Detected then begin
                Inc(DetectedMailCount);
                //* Sound
                if NotifyItemData.PlaySound
                AND (NOT AlreadyPlayingAudio)
                then begin
                    AlreadyPlayingAudio := True;
                    FormMain.PlaySound(NotifyItemData.AudioFileName, NotifyItemData.AudioLoop);
                end;
                //* Balloon
                if NotifyItemData.PopUpBalloon then begin
                    if MessageHeaders[j].SenderName <> '' then begin
                        BalloonText := BalloonText + MessageHeaders[j].SenderName + ': ' + MessageHeaders[j].Subject + #13#10;
                    end else begin
                        BalloonText := BalloonText + MessageHeaders[j].SenderAddress + ': ' + MessageHeaders[j].Subject + #13#10;
                    end;
                    Inc(DetectedBalloonMailCount);
                end;
                //* Executa a command
                if NotifyItemData.RunCommand then begin
                    ShellExecute(FormMain.Handle, 'open', PChar(NotifyItemData.RunCommandString), nil, nil, SW_SHOWNORMAL);
                end;
                //*
                if NotifyItemData.SendEMail then begin
                    if MessageHeaders[j].SenderName <> '' then begin
                        MailSenderName := MessageHeaders[j].SenderName;
                    end else begin
                        MailSenderName := MessageHeaders[j].SenderAddress;
                    end;
                    FormEMail.SendEMail(NotifyItemData.ItemName, DateTimeToStr(Now) + #13#10 + MailSenderName + ' : ' + MessageHeaders[j].Subject);
                end;
            end;
        end;
    end;

begin
    FormMain.TurnOffTrayBlinking;
    BalloonText := '';//#13#10;
    DetectedBalloonMailCount := 0;
    DetectedMailCount := 0;
    AlreadyPlayingAudio := False;
    for i := 0 to FormMain.ListViewAccounts.Items.Count - 1 do begin
        SetLength(MessageHeaders, 0);
        case TAccountItemData(FormMain.ListViewAccounts.Items.Item[i].Data).ServerType of
            asPOP3: begin
                if NOT QueryNewEmailsPOP3(TAccountItemData(FormMain.ListViewAccounts.Items.Item[i].Data), MessageHeaders) then begin
                    Continue;
                end;
            end;
            asIMAP: begin
                if NOT QueryNewEmailsIMAP(TAccountItemData(FormMain.ListViewAccounts.Items.Item[i].Data), MessageHeaders) then begin
                    Continue;
                end;
            end;
        end;
        //* Examine global
        for k := 0 to FormMain.ListViewNotificationsGlobal.Items.Count - 1 do begin
            CheckNotifications(TNotifyItemData(FormMain.ListViewNotificationsGlobal.Items.Item[k].Data));
        end;
        //* Examine per account
        LoadNotifyItemsByGUID(NotifyItems, TAccountItemData(FormMain.ListViewAccounts.Items.Item[i].Data).GUID);
        for k := Low(NotifyItems) to High(NotifyItems) do begin
            CheckNotifications(NotifyItems[k]);
        end;
        //* Free all notification items
        for k := Low(NotifyItems) to High(NotifyItems) do begin
            NotifyItems[k].Free;
        end;
        SetLength(NotifyItems, 0);
    end;
    if BalloonText <> '' then begin
        if DetectedBalloonMailCount = 1 then begin
            FormMain.ShowBalloon('Incoming e-mail!', BalloonText);
        end else begin
            FormMain.ShowBalloon('Incoming e-mails!', BalloonText);
        end;
    end;
    //* Tray icon and balloon
    FormMain.NewEmailCount := DetectedMailCount;
    if DetectedMailCount > 0 then begin
        FormMain.TimerTrayIcon.Enabled := True;
    end;
end;
*)

function TestConnectionSMTP(AccountItemData: TAccountItemData): Boolean;
var
    IdSMTP: TIdSMTP;
    SSL: TIdSSLIOHandlerSocketOpenSSL;
begin
    //Result := False;
    SSL := nil;
    IdSMTP := TIdSMTP.Create;
    if AccountItemData.Security = asSSLTLS then begin
        SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
    end;
    try
        //setup SMTP
        IdSMTP.Host := AccountItemData.HostName;
        IdSMTP.Port := AccountItemData.Port;

        IdSMTP.Username := AccountItemData.LoginName;
        IdSMTP.Password := AccountItemData.Password;

        if AccountItemData.Security = asSSLTLS then begin
            SSL.Host := IdSMTP.Host;
            SSL.Port := IdSMTP.Port;
            SSL.Destination := SSL.Host + ':' + IntToStr(SSL.Port);
            SSL.SSLOptions.Method := AccountItemData.SSLMethod; //sslvSSLv3; {good} // sslvTLSv1_2 for 3delite.hu
            IdSMTP.IOHandler := SSL;
            IdSMTP.UseTLS := utUseImplicitTLS;
        end;

        IdSMTP.Connect;
        IdSMTP.DisconnectNotifyPeer;
        Result := True;
    finally
        FreeAndNil(IdSMTP);
        if Assigned(SSL) then begin
            FreeAndNil(SSL);
        end;
    end;
end;

procedure SendEmail(AccountItemData: TAccountItemData; From: String; Recipient: String; Subject: String; Body: String);
var
    IdMessage: TIdMessage;
    IdSMTP: TIdSMTP;
    SSL: TIdSSLIOHandlerSocketOpenSSL;
begin
    SSL := nil;
    IdSMTP := TIdSMTP.Create;
    IdMessage := TIdMessage.Create;
    if AccountItemData.Security = asSSLTLS then begin
        SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
    end;
    try
        //setup SMTP
        IdSMTP.Host := AccountItemData.HostName;
        IdSMTP.Port := AccountItemData.Port;

        IdSMTP.Username := AccountItemData.LoginName;
        IdSMTP.Password := AccountItemData.Password;

        if AccountItemData.Security = asSSLTLS then begin
            SSL.Host := IdSMTP.Host;
            SSL.Port := IdSMTP.Port;
            SSL.Destination := SSL.Host + ':' + IntToStr(SSL.Port);
            SSL.SSLOptions.Method := AccountItemData.SSLMethod; //sslvSSLv3; {good} // sslvTLSv1_2 for 3delite.hu
            IdSMTP.IOHandler := SSL;
            IdSMTP.UseTLS := utUseImplicitTLS;
        end;

        //setup mail message
        IdMessage.From.Address := From;
        //if Trim(EditCC.Text) = '' then begin
            IdMessage.Recipients.EMailAddresses := Recipient;
        //end else begin
        //    IdMessage.Recipients.EMailAddresses := EditTo.Text + ',' + EditCC.Text;
        //end;
        IdMessage.Subject := Subject;
        IdMessage.Body.Text := Body;
        //if FileExists(ledAttachment.Text) then
        //    TIdAttachment.Create(MailMessage.MessageParts, ledAttachment.Text);
        //send mail
        try
            try
                IdSMTP.Connect;
                IdSMTP.Send(IdMessage);
            except on E:Exception do
                MessageDlg('ERROR: ' + E.Message, mtWarning, [mbCancel], 0);
            end;
        finally
            if IdSMTP.Connected then begin
                IdSMTP.Disconnect;
            end;
        end;
    finally
        FreeAndNil(IdMessage);
        FreeAndNil(IdSMTP);
        if Assigned(SSL) then begin
            FreeAndNil(SSL);
        end;
    end;
end;

function QueryNewEmailsPOP3(AccountItemData: TAccountItemData; Emails: TList<TIdMessage>): Boolean; overload;
var
    POP3: TIDPOP3;
    SSL: TIdSSLIOHandlerSocketOpenSSL;
    MsgCnt: Integer;
    i: Integer;
    Msg: TIdMessage;
    //Text: String;
begin
    Result := False;
    SSL := nil;
    POP3 := TIDPOP3.Create;
    if AccountItemData.Security = asSSLTLS then begin
        SSL := TIdSSLIOHandlerSocketOpenSSL.Create;
    end;
    try
        try
            //* Params
            POP3.ConnectTimeout := 10000;
            POP3.ReadTimeout := 10000;
            //* Server & login
            POP3.Host := AccountItemData.HostName;
            POP3.Port := AccountItemData.Port;
            POP3.Username := AccountItemData.LoginName;
            POP3.Password := AccountItemData.Password;
            //* SSL
            if AccountItemData.Security = asSSLTLS then begin
                SSL.Host := POP3.Host;
                SSL.Port := POP3.Port;
                SSL.Destination := SSL.Host + ':' + IntToStr(SSL.Port);
                SSL.SSLOptions.Method := AccountItemData.SSLMethod; //sslvSSLv3; // sslvTLSv1_2
                POP3.IOHandler := SSL;
                POP3.UseTLS := utUseImplicitTLS;
            end;
            // Try to connect
            POP3.Connect;
            //* Are there new messages?
            MsgCnt := POP3.CheckMessages;
            //* Parse the message headers
            for i := 1 to MsgCnt do begin
                Msg := TIdMessage.Create;
                POP3.RetrieveHeader(i, Msg);
                //* Do something with it
                Emails.Add(Msg);
            end;
            POP3.DisconnectNotifyPeer;
            if Emails.Count > 0 then begin
                Result := True;
            end;
            //* DEBUG
            //Showmessage(Text);
        finally
            if Assigned(SSL) then begin
                FreeAndNil(SSL);
            end;
            FreeAndNil(POP3);
        end;
    except
        //* ???
    end;
end;

function QueryNewEmailsIMAP(AccountItemData: TAccountItemData; Emails: TList<TIdMessage>): Boolean; overload;
var
    IMAPClient: TIdIMAP4;
    OpenSSLHandler: TIdSSLIOHandlerSocketOpenSSL;
    MsgIndex: Integer;
    MsgObject: TIdMessage;
    //PartIndex: Integer;
begin
    Result := False;
    OpenSSLHandler := nil;
    IMAPClient := TIdIMAP4.Create(nil);
    try
        if AccountItemData.Security = asSSLTLS then begin
            OpenSSLHandler := TIdSSLIOHandlerSocketOpenSSL.Create(nil);
        end;
        try
            if AccountItemData.Security = asSSLTLS then begin
                OpenSSLHandler.SSLOptions.Method := sslvSSLv3;
                IMAPClient.IOHandler := OpenSSLHandler;
                IMAPClient.UseTLS := utUseImplicitTLS;
            end;
            IMAPClient.Host := AccountItemData.HostName;
            IMAPClient.Port := AccountItemData.Port;
            IMAPClient.Username := AccountItemData.LoginName;
            IMAPClient.Password := AccountItemData.Password;
            IMAPClient.Connect;
            try
                if IMAPClient.SelectMailBox('INBOX') then begin
                    for MsgIndex := 1 to IMAPClient.MailBox.TotalMsgs do begin
                        MsgObject := TIdMessage.Create(nil);
                        IMAPClient.RetrieveHeader(MsgIndex, MsgObject);
                        //* Do something with it
                        Emails.Add(MsgObject);
                    end;
                    if Emails.Count > 0 then begin
                        Result := True;
                    end;
                end;
            finally
                IMAPClient.Disconnect;
            end;
        finally
            if Assigned(OpenSSLHandler) then begin
                FreeAndNil(OpenSSLHandler);
            end;
        end;
    finally
        FreeAndNil(IMAPClient);
    end;
end;

end.
